<?php
/*
Plugin Name: WP RPG
Plugin URI: http://wordpress.org/extend/plugins/wprpg/
Version: 1.0.6
Author: <a href="http://tagsolutions.tk">Tim G.</a>
Description: RPG Elements added to WP
Text Domain: wp-rpg
License: GPL3
*/

if ( !class_exists( 'wpRPG' ) ) {
    /**
     * wpRPG
     */
	class wpRPG {
		
		/**
		 * Consrtuctor for wpRPG Class
		 * @since 1.0.0
		 */
        public function __construct( ) {
			global $current_user;
            add_action( 'wp_footer', array(
                 $this,
                'wpRPG_check_cron' 
            ) );
            $this->file_name    = __FILE__;
            $this->plug_version = '1.0.6';
            $this->plug_slug    = basename( dirname( __FILE__ ) );
            $this->wpRPG_load_shortcodes();
			$this->crons = array(
			);
            $this->default_tabs = array(
                 'homepage' => 'Home',
                'pages' => 'Pages',
                'cron' => 'Cron Info',
				'hpreplenish' => 'HP Replenish'
            );
            
        }
        
		/**
		 * Updates player's last active time
		 * @return DBQuery or False on failure
		 * @since 1.0.0
		 */
        public function updateLastActive( ) {
            global $wpdb, $current_user;
            if ( !$current_user->ID ) {
                return false;
            }
            if ( $this->wpRPG_is_playing( $current_user->ID ) ) {
                $last_active = $wpdb->get_var( "SELECT last_active from " . $wpdb->prefix . "rpg_usermeta WHERE pid = $current_user->ID" );
                if ( $last_active <= ( time() - 300 ) )
                    return $wpdb->query( "UPDATE " . $wpdb->prefix . "rpg_usermeta SET last_active = " . time() . " WHERE pid = $current_user->ID" );
            }
        }
        
		/**
		 * Get's online status of player by ID
		 * @param int $uid PlayerID
		 * @return bool Checks against last active within 5minutes
		 * @since 1.0.0
		 */
        public static function getOnlineStatus( $uid ) {
            global $wpdb;
            $time = time() - ( 60 * 5 );
            return (bool) $wpdb->get_row( "SELECT * FROM " . $wpdb->prefix . "rpg_usermeta WHERE pid = $uid AND last_active > " . $time );
        }
        
		/**
		 * Get all players that are playing
		 * @return bool
		 * @since 1.0.0
		 */
        public function wpRPG_is_playing( $uid ) {
            global $wpdb;
            $sql = "SELECT xp, hp FROM " . $wpdb->prefix . "rpg_usermeta WHERE pid = %d";
            if ( $wpdb->get_row( $wpdb->prepare( $sql, $uid ) ) != null ) {
                return true;
                exit;
            }
            return false;
        }
        
		/**
		 * Get users by their ID
		 * @return bool|object Returns either boolean if no user is found or object of the user
		 * @since 1.0.0
		 */
        public static function wpRPG_get_user_by_id( $id ) {
            global $wpdb;
            $sql = "Select id, user_login from " . $wpdb->base_prefix . "users where id = %d";
            if ( $wpdb->get_row( $wpdb->prepare( $sql, $id ) ) != null ) {
                return $wpdb->get_row( $wpdb->prepare( $sql, $id ) );
            }
            return false;
        }
        
		/**
		 * Get player's rank by their id
		 * @return int Rank
		 * @since 1.0.0
		 */
        public static function wpRPG_player_rank( $id ) {
            global $wpdb;
            $sql_count = "SELECT * FROM " . $wpdb->prefix . "rpg_usermeta ORDER BY xp DESC";
            $res       = $wpdb->get_results( $sql_count );
            $rank      = 1;
            foreach ( $res as $item ) {
                if ( $item->id == $id ) {
                    return $rank;
                }
                ++$rank;
            }
            return 1;
        }
        
		/**
		 * Get player's level title by their level integer
		 * @param int $level
		 * @return string title
		 * @since 1.0.0
		 */
        public static function wpRPG_player_level( $level ) {
            global $wpdb;
            $sql    = "SELECT title FROM " . $wpdb->prefix . "rpg_levels l WHERE l.group='wpRPG_player_levels' AND l.min <= " . $level . " Order By l.min DESC";
            $result = $wpdb->get_results( $sql );
            return $result[ 0 ]->title;
        }
		
		/**
		 * Replenish player's HP by x points
		 * @since 1.0.2
		 * @version 1.0.6
		 * @changelog added variable increment
		 */
        public function wpRPG_replenish_hp( ) {
            global $wpdb;
            $wpdb->show_errors();
			$hpinc = get_option('wpRPG_HP_Replenish_Increment');
            $sql = "UPDATE " . $wpdb->prefix . "rpg_usermeta SET hp=hp+".$hpinc." WHERE hp<100";
            $wpdb->query( $sql );
			$sql2 = "UPDATE " . $wpdb->prefix . "rpg_usermeta SET hp=hp-(hp-100) WHERE hp>100";
			$wpdb->query( $sql2);
        }
        
		/**
		 * Check the Cron and Execute when needed
		 * @since 1.0.2
		 */
        public function wpRPG_check_cron( ) {
			foreach($this->get_crons() as $cron => $info)
			{
				$last = get_option( 'wpRPG_last_'.$cron );
				if ( !empty( $last ) && $this->wpRPG_time_elapsed( $last, $info['duration'] ) ) {
					$i  = 1;
					$xs = $this->wpRPG_time_elapsed( $last, $info['duration'] );
					while ( $i++ < $xs ) {
						if( class_exists( $info['class'] ) )
						{
							$class = new $info['class'];
							if( method_exists( $class, $info['func'] ) )
							{
								$class->$info['func']();
							}
						}
					}
					$next_t = ( time() - ( time() % $info['duration'] ) ) + $info['duration'];
					update_option( 'wpRPG_last_'.$cron, time() );
					update_option( 'wpRPG_next_'.$cron, $next_t );//$next_t
				}
			}
        }
        
		/**
		 * Get all the crons that are added
		 * @return array 
		 * @since 1.0.3
		 */
		public function get_crons()
		{
			$crons       = $this->crons;
            $new_crons = apply_filters( 'wpRPG_add_crons', $crons );
            return $new_crons;
		}
		
		/**
		 * Get the time that's elapsed by the seconds
		 * @return int 
		 * @param int $secs
		 * @since 1.0.0
		 */
        public function wpRPG_time_elapsed( $secs, $duration ) {
            return round( ( time() - $secs ) / ( $duration ) );
        }
        
		/**
		 * Load shortcodes
		 * Intentionally left empty
		 * @since 1.0.0
		 */
        public function wpRPG_load_shortcodes( ) {
            
        }
        
        /**
		 * Execute these functions during Plugin Execution 
		 * @since 1.0.0
		 */
        public function wpRPG_on_activation( ) {
            if ( !current_user_can( 'activate_plugins' ) )
                return;
            $plugin = isset( $_REQUEST[ 'plugin' ] ) ? $_REQUEST[ 'plugin' ] : '';
            check_admin_referer( "activate-plugin_{$plugin}" );
            $this->wpRPG_check_tables();
            $this->wpRPG_get_current_users_activated();
            return;
        }
        
		/**
		 * Execute these function during Plugin Deactivation
		 * @since 1.0.0
		 */
        public function wpRPG_on_deactivation( ) {
            if ( !current_user_can( 'activate_plugins' ) )
                return;
            $plugin = isset( $_REQUEST[ 'plugin' ] ) ? $_REQUEST[ 'plugin' ] : '';
            check_admin_referer( "deactivate-plugin_{$plugin}" );
        }
        
		/**
		 * Register and create these settings
		 * @since 1.0.0
		 * @version 1.0.6
		 * @changelog Added variable hp replenish
		 */
        public function wpRPG_RegisterSettings( ) {
			foreach($this->get_crons() as $cron => $info)
			{
				if ( get_option( 'wpRPG_last_'.$cron ) == null ) {
					update_option( 'wpRPG_last_'.$cron, time() );
				} elseif ( get_option( 'wpRPG_last_'.$cron ) == false ) {
					add_option( 'wpRPG_last_'.$cron, time(), "", "yes" );
				}
				if ( get_option( 'wpRPG_next_'.$cron ) == false ) {
					add_option( 'wpRPG_next_'.$cron, ( time() - ( time() % $info['duration'] ) ) + $info['duration'], "", "yes" );
				}
				if ( get_option( 'wpRPG_status_'.$cron ) == false ) {
					add_option ( 'wpRPG_status_'.$cron, ( isset($info['status']) ? $info['status'] : true), '', 'yes' );
				}
				register_setting( 'rpg_settings', 'wpRPG_last_'.$cron );
				register_setting( 'rpg_settings', 'wpRPG_next_'.$cron );
			}
			
            add_option( 'show_wpRPG_Version_footer', "0", "", "yes" );
            register_setting( 'rpg_settings', 'show_wpRPG_Version_footer' );
            add_option( 'wpRPG_rpg_installed', "0", "", "yes" );
            register_setting( 'rpg_settings', 'show_wpRPG_Version_footer' );
            register_setting( 'rpg_settings', 'wpRPG_rpg_installed' );
			/*NEW 1.0.6*/
			add_option( 'wpRPG_HP_Replenish_Increment', "1", "", "yes");
			register_setting( 'rpg_settings', 'wpRPG_HP_Replenish_Increment' );
			//
        }
        
		/**
		 * Save the settings sent by the Options page
		 * @since 1.0.0
		 */
        public function wpRPG_SaveSettings( ) {
			$options = wp_load_alloptions();
            foreach ( $_POST as $group => $setting ) {
				if ( array_key_exists($group, $options)) {
                    update_option( $group, $setting );
                }
            }
            
        }
        
		/**
		 * Check the tables and install when need be
		 * @return bool True
		 * @todo Test these tables to make sure they're up to snuff for this version
		 * @since 1.0.0
		 */
        public function wpRPG_check_tables( ) {
            global $wpdb;
            $sql = "CREATE TABLE IF NOT EXISTS " . $wpdb->prefix . "rpg_usermeta (
																	`id` int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
																	`pid` int(11) unsigned NOT NULL,
																	`last_active` int(11) unsigned default '0',
																	`xp` int(11) unsigned default '0',
																	`hp` int(11) unsigned default '20',
																	`defense` int(11) unsigned NOT NULL default '10',
																	`strength` int(11) unsigned default '5',
																	`gold` int(11) unsigned default '500',
																	`bank` int(11) unsigned default '0',
																	`race` int(11) unsigned default '1'
																	)";
            $wpdb->query( $sql );
            $sql = "CREATE TABLE IF NOT EXISTS " . $wpdb->prefix . "rpg_levels (
																	`id` int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
																	`min` int(11) unsigned NOT NULL DEFAULT '0',
																	`max` int(11) unsigned NOT NULL DEFAULT '100',
																	`group` varchar(50) NOT NULL DEFAULT '',
																	`title` varchar(50) NOT NULL DEFAULT ''
																	)";
            $wpdb->query( $sql );
			$sql = "CREATE TABLE IF NOT EXISTS " . $wpdb->prefix . "rpg_races (
																	`id` int(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
																	`title` varchar(50) NOT NULL,
																	`strength` int(11) NOT NULL DEFAULT '0',
																	`gold` int(11) NOT NULL DEFAULT '0',
																    `defense` int(11) NOT NULL DEFAULT '0'
																	)";
            $wpdb->query( $sql );
            $this->wpRPG_default_levels();
			$this->default_races();
            return true;
        }
        
		/**
		 * Create the default races during install
		 * @since 1.0.1
		 */
        public function default_races( ) {
            global $wpdb;
            $wpdb->query( "SELECT * FROM " . $wpdb->prefix . "rpg_races" );
            if ( !$wpdb->num_rows ) {
                $sql = "INSERT INTO " . $wpdb->prefix . "rpg_races (`id` ,`title` ,`strength` ,`gold` ,`defense`)VALUES ( NULL ,  'human',  '10',  '5',  '10'), ( NULL ,  'orc',  '15',  '0',  '10'), ( NULL ,  'Elf',  '10',  '10',  '5')";
                $wpdb->query( $sql );
            }
        } 		
		/**
		 * Get the rpg_meta for player
		 * @return object
		 * @param int|string $user
		 * @since 1.0.4
		 */
		public function get_player_meta( $user ) {
			global $wpdb;
			$username='';
			if(!is_numeric($user)){
				$username = $user;
				$user = 0;
			}
			$sql = "SELECT um.*, u.* FROM " . $wpdb->prefix . "rpg_usermeta um JOIN " . $wpdb->base_prefix . "users u WHERE um.pid=u.id AND u.id=" .$user . " OR u.user_nicename='" . $username . "'";
            
			return $wpdb->get_results( $sql );
		}
		
		/**
		 * Create the default levels during install
		 * @since 1.0.0
		 */
        public function wpRPG_default_levels( ) {
            global $wpdb;
            $wpdb->query( "SELECT * FROM " . $wpdb->prefix . "rpg_levels" );
            if ( !$wpdb->num_rows ) {
                $sql = "INSERT INTO " . $wpdb->prefix . "rpg_levels (`id` ,`min` ,`max` ,`group` ,`title`)VALUES ( NULL ,  '0',  '99',  'wpRPG_player_levels',  '1'), ( NULL ,  '100',  '199',  'wpRPG_player_levels',  '2'), ( NULL ,  '200',  '299',  'wpRPG_player_levels',  '3')";
                $wpdb->query( $sql );
            }
        }
        
		/**
		 * Check for the column of the db
		 * @deprecated 1.0.0
		 */
        public function wpRPG_check_column( $table, $col_name ) {
            global $wpdb;
            if ( $table != null ) {
                $results = $wpdb->get_results( "DESC $table" );
                if ( $results != null ) {
                    foreach ( $results as $row ) {
                        if ( $row->Field == $col_name ) {
                            return true;
                        }
                    }
                    return false;
                }
                return false;
            }
            return false;
        }
        
		/**
		 * Get player's that are currently not playing and make them players
		 * @since 1.0.1
		 */
        public function wpRPG_get_current_users_activated( ) {
            global $wpdb;
			$rpg_reg = new wpRPG_Registration; //Added in 1.0.1
            //$wpdb->show_errors(); // Removed in 1.0.1
            $user_ids = $wpdb->get_col( "SELECT ID FROM $wpdb->users" );
            foreach ( $user_ids as $id ) {
                if ( $wpdb->get_row( "SELECT * FROM " . $wpdb->prefix . "rpg_usermeta WHERE pid = " . $id ) == null ) {
					$rpg_reg->user_register( $id );
                }
            }
        }
        
		/**
		 * Initialize the Settings Page
		 * @since 1.0.0
		 */
        public function wpRPG_settings_page_init( ) {
            $settings_page = add_menu_page( 'Wordpress RPG Options', ' WP-RPG', 'manage_options', 'wpRPG_menu', array(
                 $this,
                'wpRPG_settings_page' 
            ) );
            add_action( "load-{$settings_page}", array(
                 $this,
                'wpRPG_load_settings_page' 
            ) );
        }
        
		/**
		 * Render the settings page and handle the Settings Submission Requests
		 * @since 1.0.0
		 */
        public function wpRPG_load_settings_page( ) {
            if ( isset( $_POST[ "wpRPG-settings-submit" ] ) && $_POST[ "wpRPG-settings-submit" ] == 'Y' ) {
                check_admin_referer( "wpRPG-settings-page" );
				//wp_die('hi');
                $this->wpRPG_SaveSettings();
                $url_parameters = isset( $_GET[ 'tab' ] ) ? 'updated=true&tab=' . $_GET[ 'tab' ] : 'updated=true';
                wp_redirect( admin_url( 'admin.php?page=wpRPG_menu&' . $url_parameters ) );
                exit;
            }
        }
        
		/**
		 * Get the Admin Tab headers
		 * @since 1.0.0
		 */
        public function wpRPG_get_admin_tab_header( ) {
            $tabs       = $this->default_tabs;
            $admin_tabs = apply_filters( 'wpRPG_add_admin_tab_header', $tabs );
            return $admin_tabs;
        }
        
		/**
		 * Grab the initial default tabs
		 * @param string $tab Name of the tab being grabbed
		 * @todo Make this be external templates
		 * @since 1.0.0
		 */
        public function wprpg_default_tabs( $tab ) {
			$content_block_starts = '<tr><td>';
			$content_block_ends = '</td></tr>';
            switch ( $tab ) {
                case 'cron':
                    $html = "<tr>";
                    $html .= "<td>";
                    $html .= "<h3>Cron Information</h3>";
                    $html .= "</td>";
                    $html .= "</tr>";
					$html .= "<table class='form-table' border=1>";
					$html .= "<tr><th>Name</th><th>Last Execution</th><th>Pending Executions</th><th>Next Execution</th><th>Actions</th>";
                    foreach($this->get_crons() as $cron => $info)
					{
					$html .= "<tr>";
                    $html .= "<td>";
                    $html .= "<h4 class='description'>$cron Cron:</h4>";
                    $html .= "</td>";
                    $html .= "<td>";
                    $html .= "<span> " . date( 'Y-m-d H:i:s', get_option( 'wpRPG_last_'.$cron ) ) . "</span>";
                    $html .= "</td>";
                    $html .= "<td>";
                    $html .= "<span> " . $this->wpRPG_time_elapsed( get_option( 'wpRPG_last_'.$cron ), $info['duration'] ) . "</span>";
                    $html .= "</td>";
                    $html .= "<td>";
                    $html .= "<span> " . date( 'Y-m-d H:i:s', get_option( 'wpRPG_next_'.$cron ) ) . "</span>";
                    $html .= "</td>";
					$html .= "<td>";
                    $html .= "<span> <input type='hidden' name='wpRPG_status_$cron' id='wpRPG_status_$cron' value='0' />
										Enabled?: <input name='wpRPG_status_$cron' id='wpRPG_status_$cron' type='checkbox' " . 
										(get_option( 'wpRPG_status_'.$cron )? 'checked':'') . " value=1></span>";
                    $html .= "</td>";
                    $html .= "</tr>";
					}
					$html .= "</table>";
					$html .= date('Y-m-d H:i:s');
                    return $html;
                    break;                    
                case 'homepage':
                    $html = "<tr>";
                    $html .= "<td>";
                    $html .= "<h3>Welcome to Wordpress RPG!</h3>";
                    $html .= "</td>";
                    $html .= "</tr>";
                    $html .= "<tr>";
                    $html .= "<td>";
                    $html .= "<span class='description'>Total Players:</span>";
                    $users = count_users();
                    $html .= "<span>" . $users[ 'total_users' ] . "</span>";
                    $html .= "</td>";
                    $html .= "</tr>";
					$html .= "<tr>";
					$html .= "<td>";
					$html .= "Thank you for using Wordpress RPG!<br/>";
					$html .= "As you can see, this is still a work in progress with alot to be done. While this is a working plugin, it's not finalized as of yet, but growing more and more capable by each version.";
					$html .= "<h4>Todo immediately:</h4>";
					$html .= "<ul><li>- Actions are coming more and more to light. Create a tab to list all actions, filters, and shortcodes available</li>
					<li>- Add a tab for Race edits and expand on Race bonuses</li>
					<li>- Add options to disable the internal plugins like Registration/Profile/Members, or create an option to allow admin to select what to use as default plugin. (ie. Use default Profile, wpRPG default, or External Profile Plugin).</li></ul>";
					$html .= "</td>";
					$html .= "</tr>";					
					$html .= "<tr>";
					$html .= "<td>";
					$html .= "<span class='description'>Show link love?:</span>";
					$html .= "<span><input type='hidden' name='show_wpRPG_Version_footer' id='show_wpRPG_Version_footer' value='0' />
									<input type='checkbox' " . ( get_option ( 'show_wpRPG_Version_footer' ) ? 'checked ' : '' ) . " name='show_wpRPG_Version_footer' id='show_wpRPG_Version_footer' value='1'>We'd appreciate it!<br></span>";
					$html .= "</td>";
					$html .= "</tr>";
                    return $html;
                    break;
                case 'pages':
                    $html = "<tr>";
                    $html .= "<td>";
                    $html .= "<h3>Pages!</h3>";
                    $html .= "</td>";
                    $html .= "</tr>";
                    $html .= "<tr>";
                    $html .= "<td>";
                    $html .= "<table border=1><tr><th>Module Page</th><th>Wordpress Page</th></tr>";
                    $html .= "<tr><td>Profile</td><td>" . $this->wpRPG_get_pages_select_html() . "</td></tr>";
                    $html .= "</table>";
                    $html .= "</td>";
                    $html .= "</tr>";
                    return $html;
                    break;
				case 'hpreplenish':
					$html = "<tr>";
                    $html .= "<td>";
                    $html .= "<h3>HP Replenish!</h3>";
                    $html .= "</td>";
                    $html .= "</tr>";
                    $html .= "<tr>";
                    $html .= "<td>";
                    $html .= "<table border=1><tr><th>Setting Name</th><th>Setting</th></tr>";
					$hpinc = get_option('wpRPG_HP_Replenish_Increment');
                    $html .= "<tr><td>HP Increment Per Cron:</td><td><input type=text value=$hpinc name='wpRPG_HP_Replenish_Increment' id='wpRPG_HP_Replenish_Increment' /></td></tr>";
                    $html .= "</table>";
                    $html .= "</td>";
                    $html .= "</tr>";
                    return $html;
                    break;
            }
        }
		
		/**
		 * Gets the correct template
		 * @return string path to template
		 * @param string $file filename of template
		 * @since 1.0.4
		 */
		public function render ( $file )
		{
			if (file_exists(get_stylesheet_directory_uri() . '/'. $file))
			{
				return get_stylesheet_directory_uri() . '/'. $file ;
			} else {
				return 'templates/'. $file ;
			}
		}
		
		/**
		 * HTML <Select> function for the Pages option | Not used elsewhere
		 * @return string $html HTML code
		 * @todo remove from core and/or make more generic
		 * @since 1.0.0
		 */
        public function wpRPG_get_pages_select_html( ) {
            $html  = "<select name='wpRPG_Profile_Page'>";
            $pages = $this->wpRPG_get_pages();
            foreach ( $pages as $page => $title ) {
                $html .= "<option name='" . $title[ 'id' ] . "' value='" . $title[ 'id' ] . "' " . ( get_option( 'wpRPG_Profile_Page' ) == $title[ 'id' ] ? 'selected=1' : '' ) . ">" . $title[ 'title' ] ."</option>";
            }
            $html .= "</select>";
            return $html;
        }
        
		/**
		 * Get all of the pages in Wordpress
		 * @return array 
		 * @since 1.0.0
		 */
        public static function wpRPG_get_pages( ) {
            $parry = array( );
            $pages = get_pages();
            foreach ( $pages as $page ) {
                $new = $parry;
                
                $parry = array_merge( $new, array(
                     array(
                         'title' => $page->post_title,
                        'id' => $page->ID 
                    ) 
                ) );
            }
            return $parry;
        }
        
		/**
		 * Get Admin Tabs that were made by default and by other plugins
		 * @return array
		 * @since 1.0.0
		 */
        public function wpRPG_get_admin_tabs( ) {
            $tabs       = array(
                 'cron' => $this->wprpg_default_tabs( 'cron' ),
                'homepage' => $this->wprpg_default_tabs( 'homepage' ),
                'pages' => $this->wprpg_default_tabs( 'pages' ),
				'hpreplenish' => $this->wprpg_default_tabs( 'hpreplenish' )
				
            );
            $admin_tabs = apply_filters( 'wpRPG_add_admin_tabs', $tabs );
            return $admin_tabs;
        }
        
		/**
		 * Get the current tab
		 * @param string name of the tab
		 * @return string HTML code
		 * @since 1.0.0
		 */
        public function wpRPG_admin_tabs( $current = 'homepage' ) {
            $tabs  = $this->wpRPG_get_admin_tab_header();
            $links = array( );
            echo '<div id="icon-themes" class="icon32"><br></div>';
            echo '<h2 class="nav-tab-wrapper">';
            foreach ( $tabs as $tab => $name ) {
                $class = ( $tab == $current ) ? ' nav-tab-active' : '';
                echo "<a class='nav-tab$class' href='?page=wpRPG_menu&tab=$tab'>$name</a>";
            }
            echo '</h2>';
        }
        
		/**
		 * Serve the wpRPG Settings page
		 * @return string HTML code
		 * @since 1.0.0
		 */
        public function wpRPG_settings_page( ) {
            global $pagenow;
            $settings = get_option( "wpRPG_theme_settings" );
?>

			<div class="wrap">
				<h2>wpRPG Settings</h2>

			<?php
            if ( isset( $_GET[ 'updated' ] ) && 'true' == esc_attr( $_GET[ 'updated' ] ) )
                echo '<div class="updated" ><p>wpRPG updated.</p></div>';
            
            if ( isset( $_GET[ 'tab' ] ) )
                $this->wpRPG_admin_tabs( $_GET[ 'tab' ] );
            else
                $this->wpRPG_admin_tabs( 'homepage' );
?>

				<div id="poststuff">
					<form method="post" name="options">
			<?php
            settings_fields( 'rpg-settings' );
            wp_nonce_field( "wpRPG-settings-page" );
            
            if ( $pagenow == 'admin.php' && $_GET[ 'page' ] == 'wpRPG_menu' ) {
                
                if ( isset( $_GET[ 'tab' ] ) )
                    $tab = $_GET[ 'tab' ];
                else
                    $tab = 'homepage';
                
                echo '<table class="form-table">';
                $tabs = $this->wpRPG_get_admin_tabs();
                echo $tabs[ $tab ];
                echo '</table>';
            }
?>
						<p class="submit" style="clear: both;">
							<input type="hidden" name="wpRPG-settings-submit" id="wpRPG-settings-submit" value="Y" />
							<input type="submit" class="button-primary" value="<?php
            _e( 'Save Changes' );
?>" />
						</p>
					</form>

				</div>

			</div>
			<?php
        }
    }
}

if ( !class_exists( 'wpRPG_Profiles' ) ) {
    class wpRPG_Profiles extends wpRPG {
        
        function __construct( ) {
            parent::__construct();
            add_shortcode( 'view_profile', array(
                 $this,
                'profileShortCodeVars' 
            ) );
            add_shortcode( 'permalink', array(
                 $this,
                'custom_permalink' 
            ) );
            add_action( 'wp_ajax_profile', array(
                 $this,
                'profileCallback' 
            ) );
            add_action( 'wp_ajax_nopriv_profile', array(
                 $this,
                'profileCallback' 
            ) );
            add_action( 'wp_loaded', array(
                 $this,
                'flushRules' 
            ) );
            add_filter( 'rewrite_rules_array', array(
                 $this,
                'rewriteRules' 
            ) );
            add_filter( 'query_vars', array(
                 $this,
                'insertQueryVars' 
            ) );
            add_filter( 'wpRPG_add_admin_tab_header', array(
                 $this,
                'addAdminTab_Header' 
            ) );
            add_filter( 'wpRPG_add_admin_tabs', array(
                 $this,
                'addAdminTab' 
            ) );
            if ( !is_admin() ) {
                add_action( 'wp_enqueue_scripts', array(
                     $this,
                    'includeJquery' 
                ) );
                add_action( 'wp_footer', array(
                     $this,
                    'includedJS' 
                ) );
            }
			
        }
        
		function register_settings() {
			if ( !get_option( 'wpRPG_Profile_Page' ) ) {
                add_option( 'wpRPG_Profile_Page', 'Profile', "", "yes" );
            }
			register_setting( 'rpg_settings', 'wpRPG_Profile_Page' );
		}
		
		/**
		 * Gets the "VIEWED" Player and their meta
		 * @return object Player's Meta
		 * @since 1.0.4
		 */
		function get_viewed_player() {
			global $current_user;
			$viewed = get_user_by('login',get_query_var( 'username' ));
			if(!$viewed)
			{
				$viewed = $current_user->ID;
				$player = $this->get_player_meta($viewed);
			}else{
				//wp_die(var_dump($viewed));
				$viewed = $viewed->ID;
				$player = $this->get_player_meta($viewed);
			}
			//wp_die(var_dump($player));
			return $player[0];
		}
		
		/**
		 * Adds info to profile section
		 * @return array
		 * @since 1.0.4
		 */
		function add_profile_section_top_right($actions)
		{
			global $current_user;
			$profile_tabs = array(
                 'attack' =>($current_user?'<a href="/attack/1">Attack</a>':null),
            );
            return array_merge( $actions, $profile_tabs );
        }		
		
		/**
		 * Adds info to profile section
		 * @return array
		 * @since 1.0.4
		 */
		function add_profile_section_mid_right($actions)
		{
			$player = $this->get_viewed_player();
			$profile_tabs = array(
				 'gold' =>  'Gold: '.$player->gold,
				 'hp' => 'HP: '.$player->hp
            );
            return array_merge( $actions, $profile_tabs );
        }
		
		/**
		 * Adds info to profile section
		 * @return array
		 * @since 1.0.4
		 */
		function add_profile_section_bottom_right($actions)
		{
			$profile_tabs = array(
                 
            );
            return array_merge( $actions, $profile_tabs );
        }	
		
        function addAdminTab( $tabs ) {
            $tab_page = array(
                 'profile' => $this->profileOptions( 1 ) 
            );
            return array_merge( $tabs, $tab_page );
        }
        
        function addAdminTab_Header( $tabs ) {
            $profile_tabs = array(
                 'profile' => 'Profile Settings' 
            );
            return array_merge( $tabs, $profile_tabs );
        }
        
        function profileOptions( $opt = 0 ) {
            $html = "<tr>";
            $html .= "<td>";
            $html .= "<h3>Welcome to Wordpress RPG Profile Module!</h3>";
            $html .= "</td>";
            $html .= "</tr>";
            $html .= "<tr>";
            $html .= "<td>";
            $html .= "<span class='description'>Nothing To See Here Yet</span>";
            $html .= "</td>";
            $html .= "</tr>";
            if ( !$opt )
                echo $html;
            else
                return $html;
        }
        
        function includeJquery( ) {
            wp_enqueue_script( 'jquery' );
			//jQuery ScrollTo Plugin
			wp_register_script( 'jquery-scrollto', 'http://balupton.github.io/jquery-scrollto/lib/jquery-scrollto.js' );
			//History.js
			wp_register_script( 'history.js', 'http://browserstate.github.io/history.js/scripts/bundled/html4+html5/jquery.history.js' );
			//Ajaxify
			wp_register_script( 'ajaxify', 'http://rawgithub.com/browserstate/ajaxify/master/ajaxify-html5.js' );
			//wp_enqueue_script( 'jquery-scrollto' );
			wp_enqueue_script( 'history.js' );
			//wp_enqueue_script( 'ajaxify' );
        }
        
        function profileShortCodeVars( $atts ) {
            extract( shortcode_atts( array(
                 'pid' => 0 
            ), $atts, 'view_profile' ) );
			//wp_die($pid);
            return $this->getProfile( $pid );
        }
        
		/**
		 * initializes the Profile Fields
		 * @since 1.0.4
		 */
		function get_profile_fields() {
			add_filter( 'profile_section_top_right', array(
						$this,
						'add_profile_section_top_right'
					) );
			add_filter( 'profile_section_mid_right', array(
						$this,
						'add_profile_section_mid_right'
					) );
			add_filter( 'profile_section_bottom_right', array(
						$this,
						'add_profile_section_bottom_right'
					) );
			add_filter( 'profile_section_top_left', array(
						$this,
						'add_profile_section_top_left'
					) );
			add_filter( 'profile_section_mid_left', array(
						$this,
						'add_profile_section_mid_left'
					) );
			add_filter( 'profile_section_bottom_left', array(
						$this,
						'add_profile_section_bottom_left'
					) );
			
		}
		
        public function getProfile( $player_id ) {
            global $wpdb, $current_user;
            $username = get_query_var( 'username' );
            if ( $player_id != 0 || !empty( $username ) ) {
					if(isset($username) && $player_id == 0)
					{
						$player_id= $username;
					}
				
				$res = $this->get_player_meta($player_id);
                if ( $res ) {
                    global $wp;
					$this->get_profile_fields();
					
                    $return_template = dirname( __FILE__ ) . '/templates/view_profile.php';
                    $result = include( $return_template );
                } else {
                    $result = '<div id="rpg_area">';
                    $result .= '<h1>User Not Found</h1>';
                    $result .= '</div><br/><br/>';
					if ( get_option ( 'show_wpRPG_Version_footer' ) )	{
						$result .= '<footer style="display:block;margin: 0 2%;border-top: 1px solid #ddd;padding: 20px 0;font-size: 12px;text-align: center;color: #999;">';
						$result .= 'Powered by <a href="http://tagsolutions.tk/wordpress-rpg/">wpRPG '. $this->plug_version .'</a></footer>';
					}
					return $result;
                }
            } else {
                if ( $current_user->ID != 0 ) {
                    return $this->getProfile( $current_user->ID );
                } else {
                    return 'Not Logged In';
                }
            }
        }
        
        function flushRules( ) {
			$profile = get_option( 'wpRPG_Profile_Page' );
			$permalink = untrailingslashit(substr(get_permalink($profile), strlen(get_option('home').'/')));
            $rules = get_option( 'rewrite_rules' );
            if ( !isset( $rules[ '('.$permalink.')/(.+)$' ] ) ) {
                global $wp_rewrite;
                $wp_rewrite->flush_rules();
            }
            
        }
        
        function rewriteRules( $rules ) {
			$profile = get_option( 'wpRPG_Profile_Page' );
			$permalink = untrailingslashit(substr(get_permalink($profile), strlen(get_option('home').'/')));
            $newrules                      = array( );
            $newrules[ '('.$permalink.')/(.+)$' ] = 'index.php?pagename=$matches[1]&username=$matches[2]';

            return $newrules + $rules;
        }
        
        function insertQueryVars( $vars ) {
            array_push( $vars, 'username' );
            return $vars;
        }
        
        function includedJS( ) {
            
?>
			<script type='text/javascript'>
				jQuery(document).ready(function($) 
				{
					/*$(window).bind('popstate', function() 
					{
					  $.ajax({url:location.pathname,success: function(data){
						//$('html').empty();
						alert(data);//$('body').html(data);
					  }});
					});*/
					
					$('a#view-profile').click(function(event) {
						event.preventDefault();
						var pageurl = $(this).attr('href');
						var url = '<?php echo get_bloginfo( 'wpurl' ); ?>';
						var them = $(this).attr('name');
						//History.pushState({path:pageurl}, null, url+"/profile/"+them);
						$.ajax({
							method: 'post',
							url: '<?php
            echo site_url( 'wp-admin/admin-ajax.php' );
?>',
							data: {
								'action': 'profile',
								'user': them,
								'ajax': true
							},
							success: function(data) {
								$('#rpg_area').empty();
								$('#rpg_area').html(data);
							}
						});
					});
				});
			</script>
			<?php
        }
        
        function profileCallback( ) {
			//wp_die($_POST['user']);
            echo $this->getProfile( $_POST[ 'user' ] );
            die( );
        }
		
		/**
		 * Get Admin Tabs that were made by default and by other plugins
		 * @return array
		 * @since 1.0.0
		 */
        public function init_user_profile_fields( $field ) {
            $sections = array();
			$section = apply_filters( 'profile_section_'.$field, $sections );
			return $section;
        }
        
    }
}

if ( !class_exists( 'wpRPG_Members' ) ) {
    class wpRPG_Members extends wpRPG {
        
        function __construct( ) {
            parent::__construct();
            add_shortcode( 'list_players', array(
                 $this,
                'listPlayers' 
            ) );
        }
        
        function listPlayers( ) {
            global $wpdb, $current_user;
            $sql    = "SELECT um.hp, um.xp, u.* FROM " . $wpdb->prefix . "rpg_usermeta um JOIN " . $wpdb->base_prefix . "users u WHERE um.pid=u.id";
            $res    = $wpdb->get_results( $sql );
            $result = '<div id="rpg_area"><table id="members" border=1>';
            $result .= '<tr><th>MemberName</th><th>XP</th><th>HP</th><th>Level</th>' . ( is_user_logged_in() ? '<th>Actions</th>' : '' ) . '</tr>';
            foreach ( $res as $u ) {
                $result .= '<tr id="player_' . $u->ID . '"><td><a href="" id="view-profile" name="' . $u->ID . '">' . $u->user_nicename . '</a></td><td>' . $u->xp . '</td><td>' . $u->hp . '</td><td>' . $this->wpRPG_player_level( $u->xp ) . '</td><td>';
                if ( is_user_logged_in() ) {
                    $result .= ( $u->ID != $current_user->ID ? $this->listPlayers_getLoggedIn_Actions( $u->ID ) : '' );
                }
                $result .= '</td></tr>';
            }
            $result .= '</table></div>';
            $result .= '<footer style="display:block;margin: 0 2%;border-top: 1px solid #ddd;padding: 20px 0;font-size: 12px;text-align: center;color: #999;">';
			if ( get_option ( 'show_wpRPG_Version_footer' ) )	{
				$result .= '<footer style="display:block;margin: 0 2%;border-top: 1px solid #ddd;padding: 20px 0;font-size: 12px;text-align: center;color: #999;">';
				$result .= 'Powered by <a href="http://tagsolutions.tk/wordpress-rpg/">wpRPG '. $this->plug_version .'</a></footer>';
			}
            return $result;
        }
        
        function listPlayers_getLoggedIn_Actions( $uid ) {
            $result = apply_filters( 'listPlayers_Loggedin_Actions', array(
                 '',
                $uid 
            ) );
            return $result[ 0 ];
        }
        
    }
}

if ( !class_exists( 'wpRPG_Hospital' ) ) {
    class wpRPG_Hospital extends wpRPG {
        
        function __construct( ) {
            parent::__construct();
            add_shortcode( 'wprpg_hospital', array(
                 $this,
                'showHospital' 
            ) );
            add_action( 'wp_ajax_hospital', array(
                 $this,
                'hospitalCallback' 
            ) );
            add_action( 'wp_ajax_nopriv_hospital', array(
                 $this,
                'hospitalCallback' 
            ) );
            if ( !is_admin() ) {
                add_action( 'wp_footer', array(
                     $this,
                    'includedJS' 
                ) );
            }
			add_filter( 'wpRPG_add_crons', array(
                 $this,
                'add_mycrons' 
            ) );
        }
        
        function showHospital( ) {
            global $wpdb, $current_user;
            $sql = "SELECT um.*, u.* FROM " . $wpdb->prefix . "rpg_usermeta um JOIN " . $wpdb->base_prefix . "users u WHERE um.pid=u.id AND um.hp<100 AND u.id=" . $current_user->ID;
            $res = $wpdb->get_results( $sql );
            if ( $res ) {
                $return_template = dirname( __FILE__ ) . '/templates/hospital.php';
                $result = include( $return_template );
            } else {
                $result = '<div id="rpg_area">';
                $result .= '<h1>Hospital</h1>
<table width=100% style="text-align:center;"><tr><td><h3>You\'re Already Fully Healed! Come Back When You Need Help!</h3></td></tr></table>
									</div>
									<br/>
									
								</div>';
                $result .= '</div><br/><br/>';
				if ( get_option ( 'show_wpRPG_Version_footer' ) )	{
					$result .= '<footer style="display:block;margin: 0 2%;border-top: 1px solid #ddd;padding: 20px 0;font-size: 12px;text-align: center;color: #999;">';
					$result .= 'Powered by <a href="http://tagsolutions.tk/wordpress-rpg/">wpRPG '. $this->plug_version .'</a></footer>';
				}
            }
            return $result;
        }
        
        function includedJS( ) {
            global $wpdb, $current_user;
            $sql = "SELECT um.*, u.* FROM " . $wpdb->prefix . "rpg_usermeta um JOIN " . $wpdb->base_prefix . "users u WHERE um.pid=u.id AND u.id=". $current_user->ID;
            $res = $wpdb->get_results( $sql );
?>
			<script type='text/javascript'>
				jQuery(document).ready(function($) {
					$('button#replenish-hp').click(function(event) {
						event.preventDefault();
						var them = '<?php
            echo $current_user->ID;
?>';
						var cost = '<?php
            echo ( 100 - $res[ 0 ]->hp );
?>';
						$.ajax({
							method: 'post',
							url: '<?php
            echo site_url( 'wp-admin/admin-ajax.php' );
?>',
							data: {
								'action': 'hospital',
								'user': them,
								'cost': cost,
								'ajax': true
							},
							success: function(data) {
								$('#rpg_area').empty();
								$('#rpg_area').html(data);
							}
						});
					});
				});
			</script>
			<?php
        }
        
        function buyHealthCare( $uid, $hp, $cost ) {
            global $wpdb;
            $sql = "UPDATE " . $wpdb->prefix . "rpg_usermeta SET hp=hp+$hp, gold=gold-$cost WHERE pid=$uid";
            $wpdb->query( $sql );
			$profiles    = new wpRPG_Profiles;
            $profiles->getProfile( $uid );
        }
        
        function hospitalCallback( ) {
            global $wpdb, $current_user;
            $sql = "SELECT um.*, u.* FROM " . $wpdb->prefix . "rpg_usermeta um JOIN " . $wpdb->base_prefix . "users u WHERE um.pid=u.id AND u.id=" . $current_user->ID;
            $res = $wpdb->get_results( $sql );
            if ( $res[ 0 ]->gold >= $_POST[ 'cost' ] ) {
                echo "You are now at full health!";
				$this->buyHealthCare( $res[ 0 ]->ID, 100 - $res[0]->hp, $_POST[ 'cost' ] );
                die( );
            } else {
                echo $this->showHospital();
                die( );
            }
        }

		/**
		 * Replenish player's HP by x points
		 * @since 1.0.2
		 * @version 1.0.6
		 * @changelog added variable increment
		 */
        public function wpRPG_replenish_hp( ) {
            global $wpdb;
            $wpdb->show_errors();
			$hpinc = get_option('wpRPG_HP_Replenish_Increment');
            $sql = "UPDATE " . $wpdb->prefix . "rpg_usermeta SET hp=hp+".$hpinc." WHERE hp<100";
            $wpdb->query( $sql );
			$sql2 = "UPDATE " . $wpdb->prefix . "rpg_usermeta SET hp=hp-(hp-100) WHERE hp<100";
			$wpdb->query( $sql2 );
        }
		
		/**
		 * Adds Cron to wpRPG cron
		 * @param array | $crons contains wpRPG::crons
		 * @returns array | Merge of new cron with old crons
		 * @since 1.0.3
		 */
		function add_mycrons( $crons )
		{
			$my_crons = array(
					 '30min_HPGain'=>array('class'=>'wpRPG_Hospital', 'func'=>'wpRPG_replenish_hp', 'duration'=>1800)
				);
				return array_merge( $crons, $my_crons );
		}
        
    }
}

if ( !class_exists( 'wpRPG_Registration' ) ) {
    class wpRPG_Registration extends wpRPG {
        function __construct( ) {
            parent::__construct();
			if( is_admin() ){
				add_action( 'admin_init', array( $this, 'register_settings' ) );
				add_action( 'admin_init', array( $this, 'register_hooks' ) );
			}
		}
        
		public function register_hooks() {
			if(get_option('show_race_selection'))
			{
				add_action( 'user_register', array(
					 $this,
					'user_register' 
				) );
				add_filter( 'registration_errors', array(
					 $this,
					'registration_errors' 
				), 10, 3 );
			}
			add_filter( 'wpRPG_add_admin_tab_header', array(
                 $this,
                'addAdminTab_Header' 
            ) );
            add_filter( 'wpRPG_add_admin_tabs', array(
                 $this,
                'addAdminTab' 
            ) );
		}
		
		function addAdminTab( $tabs ) {
            $tab_page = array(
                 'registration' => $this->registrationOptions( 1 ) 
            );
            return array_merge( $tabs, $tab_page );
        }
        
        function addAdminTab_Header( $tabs ) {
            $profile_tabs = array(
                 'registration' => 'Registration Settings' 
            );
            return array_merge( $tabs, $profile_tabs );
        }
        
        function registrationOptions( $opt = 0 ) {
            $html = "<tr>";
            $html .= "<td>";
            $html .= "<h3>Welcome to Wordpress RPG Registration Module!</h3>";
            $html .= "</td>";
            $html .= "</tr>";
            $html .= "<tr>";
            $html .= "<td>";
            $html .= "<span class='description'>Nothing To See Here Yet</span>";
            $html .= "</td>";
            $html .= "</tr>";
            if ( !$opt )
                echo $html;
            else
                return $html;
        }
		
		public function register_settings() {
			add_option( 'show_race_selection', "1", "", "yes" );
            register_setting( 'rpg_settings', 'show_race_selection' );
        }
		
        public function user_register( $user_id ) {
            global $wpdb;
            //$wpdb->show_errors();
			//wp_die(var_dump($user_id) . '<br />'. var_dump($_POST));
            if ( !$this->wpRPG_is_playing( $user_id ) ) {
                $wpdb->insert( $wpdb->prefix . "rpg_usermeta", array(
                     'pid' => $user_id 
                ), array(
                     '%d' 
                ) );
				$race = ( isset( $_POST[ 'race' ] ) ? $_POST['race'] : '1' );
                $wpdb->query( "UPDATE " . $wpdb->prefix . "rpg_usermeta SET race = " . $race . " WHERE pid = $user_id" );
            }
        }
        
        //1. Add a new form element...
        
        function register_form( ) {
            //$race = ( isset( $_POST[ 'race' ] ) ) ? $_POST[ 'race' ] : ''); ?>
			<p>
				<label for="race">
				<?php _e( 'Race', 'mydomain' ); ?>
				<br />
					<select id="race" name="race">
					<?php
            global $wpdb;
            $races = $wpdb->get_results( "SELECT * FROM " . $wpdb->prefix . "rpg_races" );
            foreach ( $races as $race ) { 
						?>
						<option value="<?php echo esc_attr( stripslashes( $race->id ) );?>" ><?php echo ucfirst($race->title); ?> </option>
			<?php } ?>
					</select>
				</label>
			</p>
			<?php
        }
        
        //2. Add validation. In this case, we make sure first_name is required.
        function registration_errors( $errors, $sanitized_user_login, $user_email ) {
            
            if ( empty( $_POST[ 'race' ] ) )
                $errors->add( 'race_error', __( '<strong>ERROR</strong>: You must include a first name.', 'mydomain' ) );
            
            return $errors;
        }

    }
}


//Main wpRPG plugin
$rpg         = new wpRPG;
// "External" Modules as would be found by other developers. Will be removed in later versions
$profiles    = new wpRPG_Profiles;
$members     = new wpRPG_Members;
$hospital    = new wpRPG_Hospital;
$rpgRegister = new wpRPG_Registration;

if ( is_admin() ) {
    add_action( 'admin_menu', array( $rpg, 'wpRPG_settings_page_init' ) );
    add_action( 'admin_init', array( $rpg, 'wpRPG_RegisterSettings' ) );
	add_action( 'admin_init', array( $profiles, 'register_settings' ) );
}
if ( !is_admin() ) {
    add_action( 'init', array( $rpg, 'updateLastActive' ) );
    add_action( 'register_form', array( $rpgRegister, 'register_form' ) );
}


register_activation_hook( __FILE__, array( $rpg, 'wpRPG_on_activation' ) );
register_deactivation_hook( __FILE__, array( $rpg, 'wpRPG_on_deactivation' ) );
?>